@@ -16,7 +16,7 @@ this program. If not, see <https://www.gnu.org/licenses/>.
 #ifndef ESCORT_DISPLAY_H_
 #define ESCORT_DISPLAY_H_
 
-#include "Point.h"
+#include "Rectangle.h"
 
 #include <cstdint>
 #include <list>
@@ -24,7 +24,6 @@ this program. If not, see <https://www.gnu.org/licenses/>.
 #include <vector>
 
 class Interface;
-class Rectangle;
 class Ship;
 class Sprite;
 
@@ -80,7 +79,7 @@ private:
 private:
 	mutable std::list<Icon> icons;
 	mutable std::vector<std::vector<const Ship *>> stacks;
-	mutable std::vector<Point> zones;
+	mutable std::vector<Rectangle> zones;
 
 	const Interface *element = nullptr;
 	int basicHeight = 0;
