@@ -577,6 +577,14 @@ void Files::OpenUserPluginFolder()
 
 
 
+// Open this user's save file directory in their native file explorer.
+void Files::OpenUserSavesFolder()
+{
+	OpenFolder(savePath);
+}
+
+
+
 void Files::LogErrorToFile(const string &message)
 {
 	if(!errorLog)
