@@ -51,6 +51,7 @@ this program. If not, see <https://www.gnu.org/licenses/>.
 #include "Plugins.h"
 #include "PointerShader.h"
 #include "Politics.h"
+#include "RenderBuffer.h"
 #include "RingShader.h"
 #include "Ship.h"
 #include "Sprite.h"
@@ -133,8 +134,11 @@ namespace {
 
 
 
-future<void> GameData::BeginLoad(bool onlyLoadData, bool debugMode)
+future<void> GameData::BeginLoad(bool onlyLoadData, bool debugMode, bool preventUpload)
 {
+	if(preventUpload)
+		spriteQueue.SetPreventUpload();
+
 	// Initialize the list of "source" folders based on any active plugins.
 	LoadSources();
 
@@ -196,15 +200,20 @@ void GameData::CheckReferences()
 
 
 
+void GameData::LoadSettings()
+{
+	// Load the key settings.
+	Command::LoadSettings(Files::Resources() + "keys.txt");
+	Command::LoadSettings(Files::Config() + "keys.txt");
+}
+
+
+
 void GameData::LoadShaders()
 {
 	FontSet::Add(Files::Images() + "font/ubuntu14r.png", 14);
 	FontSet::Add(Files::Images() + "font/ubuntu18r.png", 18);
 
-	// Load the key settings.
-	Command::LoadSettings(Files::Resources() + "keys.txt");
-	Command::LoadSettings(Files::Config() + "keys.txt");
-
 	FillShader::Init();
 	FogShader::Init();
 	LineShader::Init();
@@ -213,6 +222,7 @@ void GameData::LoadShaders()
 	RingShader::Init();
 	SpriteShader::Init();
 	BatchShader::Init();
+	RenderBuffer::Init();
 
 	background.Init(16384, 4096);
 }
