@@ -34,6 +34,10 @@ public:
 	int NoPersonSpawnWeight() const;
 	int NPCMaxMiningTime() const;
 	double UniversalFrugalThreshold() const;
+	double DepreciationMin() const;
+	double DepreciationDaily() const;
+	int DepreciationGracePeriod() const;
+	int DepreciationMaxAge() const;
 
 
 private:
@@ -42,6 +46,10 @@ private:
 	int noPersonSpawnWeight = 1000;
 	int npcMaxMiningTime = 3600;
 	double universalFrugalThreshold = .75;
+	double depreciationMin = 0.25;
+	double depreciationDaily = 0.997;
+	int depreciationGracePeriod = 7;
+	int depreciationMaxAge = 1000;
 };
 
 
