@@ -23,6 +23,7 @@ this program. If not, see <https://www.gnu.org/licenses/>.
 #include "GameData.h"
 #include "Government.h"
 #include "Planet.h"
+#include "Port.h"
 #include "Random.h"
 #include "Ship.h"
 #include "StellarObject.h"
@@ -493,7 +494,9 @@ const Planet *LocationFilter::PickPlanet(const System *origin, bool hasClearance
 		if(!planet.IsValid() || (planet.GetSystem() && planet.GetSystem()->Inaccessible()))
 			continue;
 		// Skip planets that do not offer special jobs or missions, unless they were explicitly listed as options.
-		if(planet.IsWormhole() || (requireSpaceport && !planet.HasSpaceport()) || (!hasClearance && !planet.CanLand()))
+		if(planet.IsWormhole()
+				|| (requireSpaceport && !planet.GetPort().HasService(Port::ServicesType::OffersMissions))
+				|| (!hasClearance && !planet.CanLand()))
 			if(planets.empty() || !planets.count(&planet))
 				continue;
 		if(Matches(&planet, origin))
