@@ -44,6 +44,7 @@ this program. If not, see <https://www.gnu.org/licenses/>.
 #include "Screen.h"
 #include "Ship.h"
 #include "ShipJumpNavigation.h"
+#include "SpriteSet.h"
 #include "SpriteShader.h"
 #include "StellarObject.h"
 #include "System.h"
@@ -202,6 +203,18 @@ namespace {
 		}
 		return pair<bool, bool>(blink, daysLeft > 0);
 	}
+
+	// Return total value of raid fleet (if any) and 60 frames worth of system danger.
+	double DangerFleetTotal(const PlayerInfo &player, const System &system, const bool withRaids)
+	{
+		double danger = system.Danger() * 60.;
+		if(withRaids)
+			for(const auto &raidFleet : system.GetGovernment()->RaidFleets())
+				danger += 10. * player.RaidFleetAttraction(raidFleet, &system) *
+					raidFleet.GetFleet()->Strength();
+		return danger;
+	}
+
 }
 
 const float MapPanel::OUTER = 6.f;
@@ -306,6 +319,7 @@ void MapPanel::Draw()
 	DrawSystems();
 	DrawNames();
 	DrawMissions();
+	DrawSelectedSystem();
 }
 
 
@@ -397,7 +411,7 @@ void MapPanel::FinishDrawing(const string &buttonCondition)
 		static const string UNKNOWN = "You have not yet mapped a route to this system.";
 		const Font &font = FontSet::Get(18);
 
-		const string &message = player.HasVisited(*selectedSystem) ? UNAVAILABLE : UNKNOWN;
+		const string &message = player.CanView(*selectedSystem) ? UNAVAILABLE : UNKNOWN;
 		Point point(-font.Width(message) / 2, Screen::Top() + 40);
 		font.Draw(message, point + Point(1, 1), black);
 		font.Draw(message, point, red);
@@ -433,14 +447,14 @@ void MapPanel::DrawMiniMap(const PlayerInfo &player, float alpha, const System *
 		// Draw the origin and destination systems, since they
 		// might not be linked via hyperspace.
 		Color color = Color(.5f * alpha, 0.f);
-		if(player.HasVisited(system) && system.IsInhabited(flagship) && gov)
+		if(player.CanView(system) && system.IsInhabited(flagship) && gov)
 			color = gov->GetColor().Additive(alpha);
 		RingShader::Draw(from, OUTER, INNER, color);
 
 		for(const System *link : system.Links())
 		{
 			// Only draw systems known to be attached to the jump systems.
-			if(!player.HasVisited(system) && !player.HasVisited(*link))
+			if(!player.CanView(system) && !player.CanView(*link))
 				continue;
 
 			// Draw the system link. This will double-draw the jump
@@ -455,7 +469,7 @@ void MapPanel::DrawMiniMap(const PlayerInfo &player, float alpha, const System *
 
 			gov = link->GetGovernment();
 			Color color = Color(.5f * alpha, 0.f);
-			if(player.HasVisited(*link) && link->IsInhabited(flagship) && gov)
+			if(player.CanView(*link) && link->IsInhabited(flagship) && gov)
 				color = gov->GetColor().Additive(alpha);
 			RingShader::Draw(to, OUTER, INNER, color);
 		}
@@ -714,6 +728,18 @@ Color MapPanel::GovernmentColor(const Government *government)
 
 
 
+Color MapPanel::DangerColor(const double danger)
+{
+	if(std::isnan(danger))
+		return *GameData::Colors().Get("map danger none");
+	else if(danger > .5)
+		return Color(.6, .4 * (2. - 2. * min(1., danger)), 0., .4);
+	else
+		return MapColor(2. * danger - 1.);
+}
+
+
+
 Color MapPanel::UninhabitedColor()
 {
 	return GovernmentColor(GameData::Governments().Get("Uninhabited"));
@@ -803,9 +829,9 @@ void MapPanel::Find(const string &name)
 	for(const auto &it : GameData::Systems())
 	{
 		const System &system = it.second;
-		if(system.IsValid() && !system.Inaccessible() && player.HasVisited(system))
+		if(system.IsValid() && !system.Inaccessible() && player.CanView(system))
 		{
-			int index = Search(it.first, name);
+			int index = Format::Search(it.first, name);
 			if(index >= 0 && index < bestIndex)
 			{
 				bestIndex = index;
@@ -822,9 +848,9 @@ void MapPanel::Find(const string &name)
 	for(const auto &it : GameData::Planets())
 	{
 		const Planet &planet = it.second;
-		if(planet.IsValid() && player.HasVisited(*planet.GetSystem()))
+		if(planet.IsValid() && player.CanView(*planet.GetSystem()))
 		{
-			int index = Search(it.first, name);
+			int index = Format::Search(it.first, name);
 			if(index >= 0 && index < bestIndex)
 			{
 				bestIndex = index;
@@ -865,15 +891,6 @@ bool MapPanel::IsSatisfied(const PlayerInfo &player, const Mission &mission)
 
 
 
-int MapPanel::Search(const string &str, const string &sub)
-{
-	auto it = search(str.begin(), str.end(), sub.begin(), sub.end(),
-		[](char a, char b) { return toupper(a) == toupper(b); });
-	return (it == str.end() ? -1 : it - str.begin());
-}
-
-
-
 bool MapPanel::GetTravelInfo(const System *previous, const System *next, const double jumpRange,
 	bool &isJump, bool &isWormhole, bool &isMappable, Color *wormholeColor) const
 {
@@ -886,7 +903,7 @@ bool MapPanel::GetTravelInfo(const System *previous, const System *next, const d
 			if(object.HasSprite() && object.HasValidPlanet()
 				&& object.GetPlanet()->IsWormhole()
 				&& player.HasVisited(*object.GetPlanet())
-				&& player.HasVisited(*previous) && player.HasVisited(*next)
+				&& player.CanView(*previous) && player.CanView(*next)
 				&& &object.GetPlanet()->GetWormhole()->WormholeDestination(*previous) == next)
 			{
 				isWormhole = true;
@@ -925,6 +942,36 @@ void MapPanel::UpdateCache()
 	cachedCommodity = commodity;
 	nodes.clear();
 
+	// Get danger level range so we can scale by it.
+	double dangerMax = 0.;
+	double dangerScale = 1.;
+	if(commodity == SHOW_DANGER)
+	{
+		// Scale danger to span [0, 1] based on known systems, without including raid fleets
+		// as those can greatly skew the range once they start having a chance of appearing,
+		// leading to silly (and not very useful) displays.
+		double dangerMin = numeric_limits<double>::max();
+		for(const auto &it : GameData::Systems())
+		{
+			const System &system = it.second;
+
+			// Only check displayed systems.
+			if(!system.IsValid() || system.Inaccessible() || !player.HasVisited(system))
+				continue;
+
+			const double danger = DangerFleetTotal(player, system, false);
+			if(danger > 0.)
+			{
+				if(dangerMax < danger)
+					dangerMax = danger;
+				if(dangerMin > danger)
+					dangerMin = danger;
+			}
+		}
+		if(dangerMax)
+			dangerScale = 1. / log(dangerMin / dangerMax);
+	}
+
 	// Draw the circles for the systems, colored based on the selected criterion,
 	// which may be government, services, or commodity prices.
 	const Color &closeNameColor = *GameData::Colors().Get("map name");
@@ -940,9 +987,10 @@ void MapPanel::UpdateCache()
 			continue;
 
 		Color color = UninhabitedColor();
-		if(!player.HasVisited(system))
+		if(!player.CanView(system))
 			color = UnexploredColor();
-		else if(system.IsInhabited(player.Flagship()) || commodity == SHOW_SPECIAL || commodity == SHOW_VISITED)
+		else if(system.IsInhabited(player.Flagship()) || commodity == SHOW_SPECIAL
+				|| commodity == SHOW_VISITED || commodity == SHOW_DANGER)
 		{
 			if(commodity >= SHOW_SPECIAL)
 			{
@@ -1000,6 +1048,14 @@ void MapPanel::UpdateCache()
 				const Government *gov = system.GetGovernment();
 				color = GovernmentColor(gov);
 			}
+			else if(commodity == SHOW_DANGER)
+			{
+				const double danger = DangerFleetTotal(player, system, true);
+				if(danger > 0.)
+					color = DangerColor(1. - dangerScale * log(danger / dangerMax));
+				else
+					color = DangerColor(numeric_limits<double>::quiet_NaN());
+			}
 			else
 			{
 				double reputation = system.GetGovernment()->Reputation();
@@ -1016,10 +1072,10 @@ void MapPanel::UpdateCache()
 					if(object.HasSprite() && object.HasValidPlanet())
 					{
 						const Planet *planet = object.GetPlanet();
-						hasSpaceport |= !planet->IsWormhole() && planet->HasSpaceport();
+						hasSpaceport |= !planet->IsWormhole() && planet->HasServices();
 						if(planet->IsWormhole() || !planet->IsAccessible(player.Flagship()))
 							continue;
-						canLand |= planet->CanLand() && planet->HasSpaceport();
+						canLand |= planet->CanLand() && planet->HasServices();
 						isInhabited |= planet->IsInhabited();
 						hasDominated &= (!planet->IsInhabited()
 							|| GameData::GetPolitics().HasDominated(planet));
@@ -1036,7 +1092,7 @@ void MapPanel::UpdateCache()
 		nodes.emplace_back(system.Position(), color,
 			player.KnowsName(system) ? system.Name() : "",
 			(&system == &playerSystem) ? closeNameColor : farNameColor,
-			player.HasVisited(system) ? system.GetGovernment() : nullptr);
+			player.CanView(system) ? system.GetGovernment() : nullptr);
 	}
 
 	// Now, update the cache of the links.
@@ -1055,9 +1111,9 @@ void MapPanel::UpdateCache()
 			if(link < system || !player.HasSeen(*link))
 			{
 				// Only draw links between two systems if one of the two is
-				// visited. Also, avoid drawing twice by only drawing in the
+				// viewable. Also, avoid drawing twice by only drawing in the
 				// direction of increasing pointer values.
-				if((!player.HasVisited(*system) && !player.HasVisited(*link)) || !link->IsValid())
+				if((!player.CanView(*system) && !player.CanView(*link)) || !link->IsValid())
 					continue;
 
 				bool isClose = (system == &playerSystem || link == &playerSystem);
@@ -1154,6 +1210,43 @@ void MapPanel::DrawTravelPlan()
 
 
 
+// Fill in the top-middle header bar that names the selected system, and indicates its distance.
+void MapPanel::DrawSelectedSystem()
+{
+	const Sprite *sprite = SpriteSet::Get("ui/selected system");
+	SpriteShader::Draw(sprite, Point(0. + selectedSystemOffset, Screen::Top() + .5f * sprite->Height()));
+
+	string text;
+	if(!player.KnowsName(*selectedSystem))
+		text = "Selected system: unexplored system";
+	else
+		text = "Selected system: " + selectedSystem->Name();
+
+	int jumps = 0;
+	const vector<const System *> &plan = player.TravelPlan();
+	auto it = find(plan.begin(), plan.end(), selectedSystem);
+	if(it != plan.end())
+		jumps = plan.end() - it;
+	else if(distance.HasRoute(selectedSystem))
+		jumps = distance.Days(selectedSystem);
+
+	if(jumps == 1)
+		text += " (1 jump away)";
+	else if(jumps > 0)
+		text += " (" + to_string(jumps) + " jumps away)";
+
+	const Font &font = FontSet::Get(14);
+	Point pos(-175. + selectedSystemOffset, Screen::Top() + .5 * (30. - font.Height()));
+	font.Draw({text, {350, Alignment::CENTER, Truncate::MIDDLE}},
+		pos, *GameData::Colors().Get("bright"));
+
+	// Reset the position of this UI element. If something is in the way, it will be
+	// moved back before it's drawn the next frame.
+	selectedSystemOffset = 0;
+}
+
+
+
 // Communicate the location of non-destroyed, player-owned ships.
 void MapPanel::DrawEscorts()
 {
@@ -1174,7 +1267,7 @@ void MapPanel::DrawEscorts()
 			if(squad.second.activeShips || squad.second.parkedShips)
 				RingShader::Draw(pos, INNER - 1.f, 0.f, squad.second.activeShips ? active : parked);
 
-			if(squad.second.outfits.size())
+			if(!squad.second.outfits.empty())
 				// Stored outfits are drawn/indicated by 8 short rays out of the system center.
 				for(int i = 0; i < 8; ++i)
 				{
@@ -1207,7 +1300,7 @@ void MapPanel::DrawWormholes()
 
 		for(auto &&link : it.second.Links())
 			if(!link.first->Inaccessible() && !link.second->Inaccessible() && p.IsInSystem(link.first)
-					&& player.HasVisited(*link.first) && player.HasVisited(*link.second))
+					&& player.CanView(*link.first) && player.CanView(*link.second))
 				arrowsToDraw.emplace_back(link.first, link.second, it.second.GetLinkColor());
 
 	}
