@@ -52,6 +52,7 @@ Projectile::Projectile(const Ship &parent, Point position, Angle angle, const We
 	weapon(weapon), targetShip(parent.GetTargetShip()), lifetime(weapon->Lifetime())
 {
 	government = parent.GetGovernment();
+	hitsRemaining = weapon->PenetrationCount();
 
 	// If you are boarding your target, do not fire on it.
 	if(parent.IsBoarding() || parent.Commands().Has(Command::BOARD))
@@ -78,6 +79,7 @@ Projectile::Projectile(const Projectile &parent, const Point &offset, const Angl
 {
 	government = parent.government;
 	targetGovernment = parent.targetGovernment;
+	hitsRemaining = weapon->PenetrationCount();
 
 	cachedTarget = TargetPtr().get();
 
@@ -283,17 +285,18 @@ void Projectile::Move(vector<Visual> &visuals, vector<Projectile> &projectiles)
 
 
 // This projectile hit something. Create the explosion, if any. This also
-// marks the projectile as needing deletion.
+// marks the projectile as needing deletion if it has run out of hits.
 void Projectile::Explode(vector<Visual> &visuals, double intersection, Point hitVelocity)
 {
-	clip = intersection;
-	distanceTraveled += dV.Length() * intersection;
 	for(const auto &it : weapon->HitEffects())
 		for(int i = 0; i < it.second; ++i)
-		{
 			visuals.emplace_back(*it.first, position + velocity * intersection, velocity, angle, hitVelocity);
-		}
-	lifetime = -100;
+	// The projectile dies if it has no hits remaining.
+	if(--hitsRemaining == 0)
+	{
+		clip = intersection;
+		lifetime = -100;
+	}
 }
 
 
@@ -306,6 +309,14 @@ double Projectile::Clip() const
 
 
 
+// Get whether the lifetime on this projectile has run out.
+bool Projectile::IsDead() const
+{
+	return lifetime <= 0;
+}
+
+
+
 // This projectile was killed, e.g. by an anti-missile system.
 void Projectile::Kill()
 {
@@ -332,9 +343,11 @@ const Weapon &Projectile::GetWeapon() const
 
 
 // Get information on how this projectile impacted a ship.
-Projectile::ImpactInfo Projectile::GetInfo() const
+Projectile::ImpactInfo Projectile::GetInfo(double intersection) const
 {
-	return ImpactInfo(*weapon, position, distanceTraveled);
+	// Account for the distance that this projectile traveled before intersecting
+	// with the target.
+	return ImpactInfo(*weapon, position, distanceTraveled + dV.Length() * intersection);
 }
 
 
@@ -443,3 +456,10 @@ double Projectile::DistanceTraveled() const
 {
 	return distanceTraveled;
 }
+
+
+
+uint16_t Projectile::HitsRemaining() const
+{
+	return hitsRemaining;
+}
