@@ -72,13 +72,9 @@ const string &Sprite::Name() const
 
 
 
-// Upload the given frames. The given buffer will be cleared afterwards.
+// Add the given frames, optionally uploading them. The given buffer will be cleared afterwards.
 void Sprite::AddFrames(ImageBuffer &buffer, bool is2x)
 {
-	// Do nothing if the buffer is empty.
-	if(!buffer.Pixels())
-		return;
-
 	// If this is the 1x image, its dimensions determine the sprite's size.
 	if(!is2x)
 	{
@@ -87,7 +83,9 @@ void Sprite::AddFrames(ImageBuffer &buffer, bool is2x)
 		frames = buffer.Frames();
 	}
 
-	AddBuffer(buffer, &texture[is2x]);
+	// Only non-empty buffers need to be added to the sprite.
+	if(buffer.Pixels())
+		AddBuffer(buffer, &texture[is2x]);
 }
 
 
@@ -107,11 +105,17 @@ void Sprite::AddSwizzleMaskFrames(ImageBuffer &buffer, bool is2x)
 // Free up all textures loaded for this sprite.
 void Sprite::Unload()
 {
-	glDeleteTextures(2, texture);
-	texture[0] = texture[1] = 0;
+	if(texture[0] || texture[1])
+	{
+		glDeleteTextures(2, texture);
+		texture[0] = texture[1] = 0;
+	}
 
-	glDeleteTextures(2, swizzleMask);
-	swizzleMask[0] = swizzleMask[1] = 0;
+	if(swizzleMask[0] || swizzleMask[1])
+	{
+		glDeleteTextures(2, swizzleMask);
+		swizzleMask[0] = swizzleMask[1] = 0;
+	}
 
 	width = 0.f;
 	height = 0.f;
