@@ -173,6 +173,8 @@ void System::Load(const DataNode &node, Set<Planet> &planets)
 			}
 			else if(key == "hidden")
 				hidden = false;
+			else if(key == "shrouded")
+				shrouded = false;
 			else if(key == "inaccessible")
 				inaccessible = false;
 			else if(key == "no raids")
@@ -188,6 +190,8 @@ void System::Load(const DataNode &node, Set<Planet> &planets)
 		// Handle the attributes without values.
 		if(key == "hidden")
 			hidden = true;
+		else if(key == "shrouded")
+			shrouded = true;
 		else if(key == "inaccessible")
 			inaccessible = true;
 		else if(key == "no raids")
@@ -636,14 +640,10 @@ bool System::Hidden() const
 
 
 
-// Return how much ramscoop is generated by this system, depending on the given ship ramscoop value.
-double System::RamscoopFuel(double shipRamscoop, double scale) const
+// Defines whether a system can be remembered when out of view.
+bool System::Shrouded() const
 {
-	// Even if a ship has no ramscoop, it can harvest a tiny bit of fuel by flying close to the star,
-	// provided the system allows it. Both the system and the gamerule must allow the universal ramscoop
-	// in order for it to function.
-	double universal = 0.05 * scale * universalRamscoop * GameData::GetGamerules().UniversalRamscoopActive();
-	return max(0., SolarWind() * .03 * scale * ramscoopMultiplier * (sqrt(shipRamscoop) + universal) + ramscoopAddend);
+	return shrouded;
 }
 
 
@@ -656,6 +656,18 @@ bool System::Inaccessible() const
 
 
 
+// Return how much ramscoop is generated by this system, depending on the given ship ramscoop value.
+double System::RamscoopFuel(double shipRamscoop, double scale) const
+{
+	// Even if a ship has no ramscoop, it can harvest a tiny bit of fuel by flying close to the star,
+	// provided the system allows it. Both the system and the gamerule must allow the universal ramscoop
+	// in order for it to function.
+	double universal = 0.05 * scale * universalRamscoop * GameData::GetGamerules().UniversalRamscoopActive();
+	return max(0., SolarWind() * .03 * scale * ramscoopMultiplier * (sqrt(shipRamscoop) + universal) + ramscoopAddend);
+}
+
+
+
 // Additional travel distance to target for ships entering through hyperspace.
 double System::ExtraHyperArrivalDistance() const
 {
